#include <SPI.h>
#include <LoRa.h>
#include <Arduino.h>
#include <Wire.h>
#include <Adafruit_SHT31.h>
#include <Adafruit_DPS310.h>

// RFW9X pin mappings on LILYGO board
#define SCK 6
#define MISO 4
#define MOSI 5
#define SS 7
#define RST 0
#define DI0 -1

const int SCL_PIN = 3;
const int SDA_PIN = 2;
const int ANEM_PIN = 8;

const int WV0 = 1;
const int WV1 = 10;
const int WV2 = 18;

Adafruit_SHT31 sht30 = Adafruit_SHT31();
Adafruit_DPS310 dps310;
Adafruit_Sensor *dps_pressure = dps310.getPressureSensor();

// Initialize wind speed variables
float wind_mps = 0;
float wind_mph = 0; 
float rev_per_time = 0;
unsigned long firstMillis = 0;
unsigned long lastMillis = 0;
unsigned long lastIntTime = 0;
int rev_counter = 0;   

void IRAM_ATTR windSpeedISR ()
{
  unsigned long intTime = millis();
  if(intTime - lastIntTime > 100)
  {
    if (rev_counter == 0)
      firstMillis = millis();
    rev_counter++;
    lastMillis = millis();
  }
  lastIntTime = intTime;
}

/* Read temp and humidity data and display it */
void readAndDisplaySensor(void *param)
{
	float humidity, temp;
	char displayStr[50];
  sensors_event_t pressure_event;
	while (1) {
		temp = sht30.readTemperature();
    humidity = sht30.readHumidity();

    if (dps310.pressureAvailable()) {
      dps_pressure->getEvent(&pressure_event);
    }

    rev_per_time = 0;
    if(rev_counter != 0)
      rev_per_time = 1.0 / ((lastMillis - firstMillis) / rev_counter);

    wind_mps = rev_per_time*1000*6.2831853*0.092; // wind in meters/sec
    wind_mph = wind_mps*2.237; 
    rev_counter = 0;

    u_int8_t wv0 = digitalRead(WV0);
    u_int8_t wv1 = digitalRead(WV1);
    u_int8_t wv2 = digitalRead(WV2);
    u_int8_t wdir = (wv0 << 2) + (wv1 << 1) + (wv2);
    
		sprintf(displayStr, "temp: %.2f\nhum: %.2f\npres: %.2f\nwspd: %.2f\nwdir: %d\n", temp, humidity, pressure_event.pressure, wind_mph, wdir);
		Serial.println(displayStr);
    LoRa.beginPacket();
    LoRa.print(displayStr);
    LoRa.endPacket();
		
		vTaskDelay(10000 / portTICK_PERIOD_MS);
	}
}

/* Make sure the sensor can be found */
void findSensor()
{
	/* Find the sensor */
	Serial.println("Looking for SHT30...");
	int retries = 0;
	while (!sht30.begin(0x44) && retries < 10) {
		Serial.println("Couldn't find SHT30");
		retries++;
		delay(500);

		if (retries == 10)
			while (1) delay(10);
	}
	Serial.println("Found SHT30 sensor");

  Serial.println("Looking for DPS310...");
  retries = 0;
	while (!dps310.begin_I2C(0x76) && retries < 10) {
		Serial.println("Couldn't find DPS310");
		retries++;
		delay(500);

		if (retries == 10)
			while (1) delay(10);
	}
	Serial.println("Found DPS310 sensor");
}

void setup() {
	Serial.begin(115200);
	Wire.begin(SDA_PIN, SCL_PIN); // set the correct scl and sda pins on our board
  
  SPI.begin(SCK,MISO,MOSI,SS);
  LoRa.setPins(SS,RST,DI0);
  if (!LoRa.begin(915e6)) {
    Serial.println("Starting LoRa failed!");
    while (1);
  }
  Serial.println("Started LoRa");

	findSensor();

  pinMode(ANEM_PIN, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(ANEM_PIN), windSpeedISR, FALLING);

  pinMode(WV0, INPUT_PULLDOWN);
  pinMode(WV1, INPUT_PULLDOWN);
  pinMode(WV2, INPUT_PULLDOWN);

	xTaskCreate(readAndDisplaySensor, "display sensor data", 16384, NULL, 1, NULL); 
	vTaskDelete(NULL);
}

void loop() {}
